matrix world;
matrix wvp;
float yLevel;

struct VS_INPUT {
	float3 pos : POSITION;
	float3 norm : TEXCOORD0;
};

struct PS_INPUT {
	float4 pos : SV_POSITION;
	float3 norm : TEXCOORD0;
	float worldPosY : TEXCOORD1;
};

PS_INPUT vsMain( VS_INPUT In ) {
	PS_INPUT Out;
	Out.worldPosY = mul( float4( In.pos, 1.0 ), world ).y;
	Out.pos = mul( float4( In.pos, 1.0 ), wvp );
	return Out;
}

float4 psMain( PS_INPUT In ) : SV_TARGET {
	return step( In.worldPosY, yLevel );
}

RasterizerState rsBackface {
	CullMode = Front;
};

RasterizerState rsFrontface {
	CullMode = Back;
};

BlendState bsBackface {
	AlphaToCoverageEnable = FALSE;
	BlendEnable[0] = TRUE;
	SrcBlend = ONE;
	DestBlend = ONE;
	BlendOp = ADD;
	SrcBlendAlpha = ONE;
	DestBlendAlpha = ZERO;
	BlendOpAlpha = ADD;
	RenderTargetWriteMask[0] = 0x0F;
};

BlendState bsFrontface {
	AlphaToCoverageEnable = FALSE;
	BlendEnable[ 0 ] = TRUE;
	SrcBlend = ONE;
	DestBlend = ONE;
	BlendOp = REV_SUBTRACT;
	SrcBlendAlpha = ONE;
	DestBlendAlpha = ZERO;
	BlendOpAlpha = ADD;
	RenderTargetWriteMask[ 0 ] = 0x0F;
};

DepthStencilState dss {
	DepthEnable = FALSE;
};

technique10 tech {
	pass BackfaceDraw {
		SetVertexShader( CompileShader( vs_4_0, vsMain() ) );
		SetPixelShader( CompileShader( ps_4_0, psMain() ) );
		
		SetRasterizerState( rsBackface );
		SetBlendState( bsBackface, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		SetDepthStencilState( dss, 0 );
	}

	pass FrontfaceDraw {
		SetVertexShader( CompileShader( vs_4_0, vsMain() ) );
		SetPixelShader( CompileShader( ps_4_0, psMain() ) );

		SetRasterizerState( rsFrontface );
		SetBlendState( bsFrontface, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		SetDepthStencilState( dss, 0 );
	}
}
